%global common_description %{expand:
Opencryptoki implements the PKCS#11 specification v2.20 for a set of
cryptographic hardware, such as IBM 4764 and 4765 crypto cards, and the
Trusted Platform Module (TPM) chip. Opencryptoki also brings a software
token implementation that can be used without any cryptographic
hardware.}

Summary:		Implementation of the PKCS#11 (Cryptoki) specification v3.0
Name:			opencryptoki
Version:		3.22.0
Release:		1%{?dist}
License:		CPL-1.0
URL:			https://github.com/opencryptoki/opencryptoki
Source0:		https://github.com/opencryptoki/%{name}/archive/v%{version}/%{name}-%{version}.tar.gz
Source1:		opencryptoki.module

Patch0001:		        Fixes-for-Marvin-attack.patch
Patch0002:	            Implement-implicit-rejection-for-RSA-PKCS#1-v1.5-de-padding.patch	
# bz#1373833, change tmpfiles snippets from /var/lock/* to /run/lock/*
Patch3000:              opencryptoki-3.11.0-lockdir.patch
# fix install problem in buildroot
Patch3001:              opencryptoki-3.21.0-p11sak.patch

BuildRequires:  gcc gcc-c++ openssl-devel openldap-devel expect make
BuildRequires:  autoconf automake libtool bison flex libcap-devel systemd-rpm-macros
Requires:       %{name}-libs = %{version}-%{release}
Requires:       %{name}(token)
Requires(pre):  %{name}-libs = %{version}-%{release}
Requires(post):	diffutils
Requires(pre):  coreutils
Requires:		(selinux-policy >= 34.9-1 if selinux-policy-targeted)
%{?systemd_requires}

%description
%{common_description}
This package contains the Slot Daemon (pkcsslotd) and general utilities.

%package libs
Summary:		The run-time libraries for opencryptoki package
Requires(pre):	shadow-utils

%description libs
%{common_description}
This package contains the PKCS#11 library implementation, and requires
at least one token implementation (packaged separately) to be fully
functional.

%package devel
Summary:		Development files for openCryptoki
Requires:		%{name}-libs = %{version}-%{release}

%description devel
%{common_description}
This package contains the development header files for building
opencryptoki and PKCS#11 based applications

%package swtok
Summary:		The software token implementation for opencryptoki
Requires(pre):	%{name}-libs = %{version}-%{release}
Requires:		%{name}-libs = %{version}-%{release}
Provides:		%{name}(token)

%description swtok
%{common_description}
This package brings the software token implementation to use opencryptoki
without any specific cryptographic hardware.

%package icsftok
Summary:		ICSF token support for opencryptoki
Requires(pre):	%{name}-libs = %{version}-%{release}
Requires:		%{name}-libs = %{version}-%{release}
Provides:		%{name}(token)

%description icsftok
%{common_description}
This package brings the necessary libraries and files to support
ICSF token in the opencryptoki stack.

%prep
%autosetup -n %{name}-%{version} -p1

%build
./bootstrap.sh
%configure --with-systemd=%{_unitdir} --enable-testcases \
    --with-pkcsslotd-user=pkcsslotd --with-pkcs-group=pkcs11 \
    --disable-tpmtok \
    --disable-icatok \
    --disable-ccatok \
    --disable-ep11tok \
    --disable-pkcsep11_migrate

%make_build CHGRP=/bin/true


%install
%make_install CHGRP=/bin/true

%pre
%global cfile /etc/opencryptoki/opencryptoki.conf
%global csuffix .rpmsave.XyoP
if test $1 -gt 1 && test -f %{cfile} ; then
    cp -p %{cfile} %{cfile}%{csuffix}
fi

%pre libs
getent group pkcs11 >/dev/null || groupadd -r pkcs11
getent passwd pkcsslotd >/dev/null || useradd -r -g pkcs11 -d /run/opencryptoki -s /sbin/nologin -c "Opencryptoki pkcsslotd user" pkcsslotd
exit 0

%post
if test $1 -gt 1 && test -f %{cfile} ; then
    if ( ! cmp -s %{cfile} %{cfile}%{csuffix} ) ; then
        cp -p %{cfile} %{cfile}.rpmnew
    fi
    cp -p %{cfile}%{csuffix} %{cfile} && rm -f %{cfile}%{csuffix}
fi

%systemd_post pkcsslotd.service
if test $1 -eq 1; then
	%tmpfiles_create %{name}.conf
fi

%preun
%systemd_preun pkcsslotd.service

%postun
%systemd_postun_with_restart pkcsslotd.service


%files
%doc ChangeLog FAQ README.md
%doc doc/opencryptoki-howto.md
%doc doc/README.token_data
%doc %{_docdir}/%{name}/*.conf
%dir %{_sysconfdir}/%{name}
%verify(not md5 size mtime) %config(noreplace) %{_sysconfdir}/%{name}/%{name}.conf
%attr(0640, root, pkcs11) %config(noreplace) %{_sysconfdir}/%{name}/p11sak_defined_attrs.conf
%attr(0640, root, pkcs11) %config(noreplace) %{_sysconfdir}/%{name}/strength.conf
%{_tmpfilesdir}/%{name}.conf
%{_unitdir}/pkcsslotd.service
%{_sbindir}/p11sak
%{_sbindir}/pkcstok_migrate
%{_sbindir}/pkcsconf
%{_sbindir}/pkcsslotd
%{_sbindir}/pkcsstats
%{_sbindir}/pkcshsm_mk_change
%{_libdir}/opencryptoki/methods
%{_libdir}/pkcs11/methods
%dir %attr(770,root,pkcs11) %{_sharedstatedir}/%{name}
%dir %attr(770,root,pkcs11) %{_sharedstatedir}/%{name}/HSM_MK_CHANGE
%ghost %dir %attr(770,root,pkcs11) %{_rundir}/lock/%{name}
%ghost %dir %attr(770,root,pkcs11) %{_rundir}/lock/%{name}/*
%dir %attr(710,pkcsslotd,pkcs11) /run/%{name}
%{_mandir}/man1/p11sak.1*
%{_mandir}/man1/pkcstok_migrate.1*
%{_mandir}/man1/pkcsconf.1*
%{_mandir}/man1/pkcsstats.1*
%{_mandir}/man1/pkcshsm_mk_change.1*
%{_mandir}/man5/policy.conf.5*
%{_mandir}/man5/strength.conf.5*
%{_mandir}/man5/%{name}.conf.5*
%{_mandir}/man5/p11sak_defined_attrs.conf.5*
%{_mandir}/man7/%{name}.7*
%{_mandir}/man8/pkcsslotd.8*

%files libs
%license LICENSE
%{_sysconfdir}/ld.so.conf.d/*
%dir %{_libdir}/opencryptoki
%{_libdir}/opencryptoki/libopencryptoki.*
%{_libdir}/opencryptoki/PKCS11_API.so
%dir %{_libdir}/opencryptoki/stdll
%dir %{_libdir}/pkcs11
%{_libdir}/pkcs11/libopencryptoki.so
%{_libdir}/pkcs11/PKCS11_API.so
%{_libdir}/pkcs11/stdll
%dir %attr(770,root,pkcs11) %{_localstatedir}/log/opencryptoki

%files devel
%{_includedir}/%{name}/
%{_libdir}/pkgconfig/%{name}.pc

%files swtok
%{_libdir}/opencryptoki/stdll/libpkcs11_sw.*
%{_libdir}/opencryptoki/stdll/PKCS11_SW.so
%dir %attr(770,root,pkcs11) %{_sharedstatedir}/%{name}/swtok/
%dir %attr(770,root,pkcs11) %{_sharedstatedir}/%{name}/swtok/TOK_OBJ/

%files icsftok
%doc doc/README.icsf_stdll
%{_sbindir}/pkcsicsf
%{_libdir}/opencryptoki/stdll/libpkcs11_icsf.*
%{_libdir}/opencryptoki/stdll/PKCS11_ICSF.so
%dir %attr(770,root,pkcs11) %{_sharedstatedir}/%{name}/icsf/
%{_mandir}/man1/pkcsicsf.1*

%changelog
* Tue Jan 30 2024 wynnfeng <wynnfeng@tencent.com> - 3.22.0-1
- update to 3.22.0 and fix CVE-2024-0914

* Fri Sep 08 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 3.21.0-2
- Rebuilt for OpenCloudOS Stream 23.09

* Wed Jul 12 2023 Shuo Wang <abushwang@tencent.com> - 3.21.0-1
- update to 3.21.0

* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 3.19.0-4
- Rebuilt for OpenCloudOS Stream 23.05

* Fri Mar 31 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 3.19.0-3
- Rebuilt for OpenCloudOS Stream 23

* Wed Dec 14 2022 Shuo Wang <abushwang@tencent.com> - 3.19.0-2
- update %{?systemd_requires}

* Thu Nov 17 2022 Shuo Wang <abushwang@tencent.com> - 3.19.0-1
- initial build
